"""
Configuration Constants

"""
APPLICATION = 'Application'
DAEMON = 'Daemon'
HTTP_SERVER = 'HTTPServer'
LOGGING = 'Logging'
ROUTES = 'Routes'

AUTOMATIC = 'automatic'
BASE = 'base'
BASE_VARIABLE = '{{base}}'
CERT_REQS = 'cert_reqs'
DEBUG = 'debug'
DEFAULT_LOCALE = 'default_locale'
FILE = 'file'
LOG_FUNCTION = 'log_function'
NEWRELIC = 'newrelic_ini'
NO_KEEP_ALIVE = 'no_keep_alive'
NONE = 'none'
OPTIONAL = 'optional'
PROCESSES = 'processes'
PATHS = 'paths'
PORTS = 'ports'
RABBITMQ = 'rabbitmq'
REDIS = 'redis'
REQUIRED = 'required'
SSL_OPTIONS = 'ssl_options'
STATIC = 'static'
TEMPLATES = 'templates'
TRANSFORMS = 'transforms'
TRANSLATIONS = 'translations'
UI_MODULES = 'ui_modules'
VERSION = 'version'
XHEADERS = 'xheaders'
